@@ -348,7 +348,6 @@ class Storage(Loadable, Drawable, Connectable, _core.Storage, metaclass=NodeMeta
     __parameter_value_attributes__ = ("initial_volume",)
 
     def __init__(self, model, name, outputs=1, inputs=1, *args, **kwargs):
-
         min_volume = pop_kwarg_parameter(kwargs, "min_volume", 0.0)
         if min_volume is None:
             min_volume = 0.0
@@ -495,6 +494,18 @@ class RollingVirtualStorage(
 ):
     """A rolling virtual storage node useful for implementing rolling licences.
 
+    The volume of the node is updated each timestep with the volume of water utilised in the timestep immediately
+    prior to the rolling period.
+
+    If the initial volume of the storage is less than the maximum volume then the parameter will calculate
+    an initial utilisation value. This is set equal to: (max volume - initial volume) / (timesteps - 1). This
+    utilisation is assumed to occurred equally across each timestep of the rolling period. The storage is replenished
+    by this value for each timestep until a full rolling period is completed. At this point, replenishment will
+    be based on the previous utilisation of the storage during the model run. Note that this changes the previous
+    default behaviour of the node up to Pywr version 1.17.1, where no initial utilisation was calculated. This meant
+    that it was impossible for the storage volume to be higher than the initial volume even if this was lower than the
+    max volume.
+
     Parameters
     ----------
     model: pywr.core.Model
@@ -509,8 +520,10 @@ class RollingVirtualStorage(
         The minimum volume the storage is allowed to reach.
     max_volume: float or parameter
         The maximum volume of the storage.
-    initial_volume: float
-        The initial storage volume.
+    initial_volume, initial_volume_pc : float (optional)
+        Specify initial volume in either absolute or proportional terms. Both are required if `max_volume`
+        is a parameter because the parameter will not be evaluated at the first time-step. If both are given
+        and `max_volume` is not a Parameter, then the absolute value is ignored.
     timesteps : int
         The number of timesteps to apply to the rolling storage over.
     days : int
@@ -533,6 +546,7 @@ class RollingVirtualStorage(
             min_volume = 0.0
         max_volume = pop_kwarg_parameter(kwargs, "max_volume", 0.0)
         initial_volume = kwargs.pop("initial_volume", 0.0)
+        initial_volume_pc = kwargs.pop("initial_volume_pc", None)
         cost = pop_kwarg_parameter(kwargs, "cost", 0.0)
         factors = kwargs.pop("factors", None)
         days = kwargs.pop("days", None)
@@ -546,6 +560,7 @@ class RollingVirtualStorage(
         self.min_volume = min_volume
         self.max_volume = max_volume
         self.initial_volume = initial_volume
+        self.initial_volume_pc = initial_volume_pc
         self.cost = cost
         self.nodes = nodes
         self.days = days
@@ -691,6 +706,44 @@ class SeasonalVirtualStorage(AnnualVirtualStorage):
                 self.active = False
 
 
+class MonthlyVirtualStorage(VirtualStorage):
+    """A virtual storage that resets after a given number of months
+
+    Parameters
+    ----------
+    months : int, default 1
+        The number of months after which the storage volume resets.
+    initial_months : int, default 0
+        The number of months into the reset period the storages is at when the model run starts.
+    reset_to_initial_volume : bool
+        Reset the volume to the initial volume instead of maximum volume each year (default is False).
+    """
+
+    def __init__(self, *args, **kwargs):
+        self.months = kwargs.pop("months", 1)
+        self.initial_months = kwargs.pop("initial_months", 0)
+        self.reset_to_initial_volume = kwargs.pop("reset_to_initial_volume", False)
+        self._count = self.initial_months - 1
+        self._last_month = None
+        super(MonthlyVirtualStorage, self).__init__(*args, **kwargs)
+
+    def reset(self):
+        super().reset()
+        self._count = self.initial_months - 1
+        self._last_month = None
+
+    def before(self, ts):
+        super().before(ts)
+        if ts.month != self._last_month:
+            self._last_month = ts.month
+            self._count += 1
+            if self._count == self.months:
+                self._count = 0
+                self._reset_storage_only(
+                    use_initial_volume=self.reset_to_initial_volume
+                )
+
+
 class PiecewiseLink(Node):
     """An extension of Node that represents a non-linear Link with a piece wise cost function.
 
@@ -1282,4 +1335,12 @@ class LossLink(Node):
         self.commit_all(self.net.flow)
 
 
+class ShadowStorage(Loadable, Drawable, _core.ShadowStorage, metaclass=NodeMeta):
+    pass
+
+
+class ShadowNode(Loadable, Drawable, _core.ShadowNode, metaclass=NodeMeta):
+    pass
+
+
 from pywr.domains.river import *  # noqa
