@@ -70,7 +70,6 @@ class TestBinaryStepParameter:
         ],
     )
     def test_bounds(self, simple_linear_model, lb, ub):
-
         m = simple_linear_model
 
         data = {
@@ -88,18 +87,21 @@ class TestBinaryStepParameter:
 
 class TestRectifierParameter:
     @pytest.mark.parametrize(
-        ["internal_value", "max_output", "upper_bounds"],
+        ["internal_value", "max_output", "upper_bounds", "min_output"],
         [
-            (-1.0, 1.0, 1.0),
-            (0.0, 1.0, 1.0),
-            (0.5, 1.0, 1.0),
-            (0.5, 10.0, 1.0),
-            (0.5, 10.0, 5.0),
-            (1.0, 10.0, 1.0),
+            (-1.0, 1.0, 1.0, 0.0),
+            (0.0, 1.0, 1.0, 0.0),
+            (0.5, 1.0, 1.0, 0.0),
+            (0.5, 10.0, 1.0, 0.0),
+            (0.5, 10.0, 5.0, 0.0),
+            (1.0, 10.0, 1.0, 0.0),
+            (0.0, 10.0, 1.0, 5.0),
+            (0.5, 10.0, 1.0, 5.0),
+            (1.0, 10.0, 1.0, 5.0),
         ],
     )
     def test_binary_step(
-        self, simple_linear_model, internal_value, max_output, upper_bounds
+        self, simple_linear_model, internal_value, max_output, upper_bounds, min_output
     ):
         """Test the binary step profile parameter."""
 
@@ -110,12 +112,16 @@ class TestRectifierParameter:
         if internal_value <= 0.0:
             expected_values = np.zeros(365)
         else:
-            expected_values = np.ones(365) * max_output * internal_value / upper_bounds
+            range = max_output - min_output
+            expected_values = np.ones(365) * (
+                min_output + range * internal_value / upper_bounds
+            )
 
         data = {
             "type": "rectifier",
             "value": internal_value,
             "max_output": max_output,
+            "min_output": min_output,
             "upper_bounds": upper_bounds,
         }
 
@@ -144,7 +150,6 @@ class TestRectifierParameter:
         ],
     )
     def test_bounds(self, simple_linear_model, lb, ub):
-
         m = simple_linear_model
 
         data = {
@@ -218,7 +223,6 @@ class TestLogisticParameter:
         ],
     )
     def test_bounds(self, simple_linear_model, lb, ub):
-
         m = simple_linear_model
 
         data = {
