@@ -2,7 +2,11 @@
 # -*- coding: utf-8 -*-
 
 import datetime
+import json
+
 import pytest
+
+import pywr.solvers
 from fixtures import *
 from helpers import *
 from pywr._core import Timestep, ScenarioIndex
@@ -14,6 +18,7 @@ from pywr.parameters import (
     ConstantParameter,
     DataFrameParameter,
     AggregatedParameter,
+    ArrayIndexedParameter,
 )
 from pywr.recorders import assert_rec, AssertionRecorder
 
@@ -394,7 +399,7 @@ def test_storage_max_volume_param():
     otpt = Output(model, "output", max_flow=99999, cost=-99999)
     storage.connect(otpt)
 
-    p = ConstantParameter(model, 20.0)
+    p = ArrayIndexedParameter(model, [20.0])
     storage.max_volume = p
     storage.initial_volume = 10.0
     storage.initial_volume_pc = 0.5
@@ -404,19 +409,15 @@ def test_storage_max_volume_param():
 
     model.run()
 
-    p.set_double_variables(
-        np.asarray(
-            [
-                40.0,
-            ]
-        )
-    )
+    p1 = ArrayIndexedParameter(model, [10.0])
+    storage.max_volume = p1
     model.reset()
 
     # This is a demonstration of the issue describe in #470
     #   https://github.com/pywr/pywr/issues/470
     # The initial storage is defined in both absolute and relative terms
     # but these are now not consistent with one another and the updated max_volume
+    # The issue has been fixed when max_volume is set to a constant parameter.
 
     np.testing.assert_allclose(storage.volume, 10.0)
     np.testing.assert_allclose(storage.current_pc, 0.5)
@@ -446,6 +447,31 @@ def test_storage_initial_volume_pc():
     np.testing.assert_allclose(storage.volume, 20.0)
 
 
+def test_storage_initial_volume_pc_constant_mx_vol_param():
+    """Test that setting initial volume as a percentage works as expected."""
+    model = Model(
+        start=pandas.to_datetime("2016-01-01"), end=pandas.to_datetime("2016-01-01")
+    )
+
+    mx_vol = ConstantParameter(model, 20.0)
+    storage = Storage(
+        model, "storage", inputs=1, outputs=0, initial_volume_pc=0.5, max_volume=mx_vol
+    )
+    otpt = Output(model, "output", max_flow=99999, cost=-99999)
+    storage.connect(otpt)
+
+    model.setup()
+    np.testing.assert_allclose(storage.current_pc, 0.5)
+    np.testing.assert_allclose(storage.volume, 10.0)
+
+    model.run()
+
+    storage.max_volume = 40.0
+    model.reset()
+    np.testing.assert_allclose(storage.current_pc, 0.5)
+    np.testing.assert_allclose(storage.volume, 20.0)
+
+
 def test_storage_initial_missing_raises():
     """Test that a RuntimeError is raised if no initial volume is specified."""
     model = Model()
@@ -631,3 +657,107 @@ def test_timestep_days_in_year_methods(
     ts = simple_linear_model.timestepper.current
     assert days_in_current_year == ts.days_in_current_year()
     assert days_in_next_year == ts.days_in_next_year()
+
+
+class TestSpecifyingSolver:
+    @pytest.mark.parametrize("solver", ["null", "glpk", "glpk-edge"])
+    def test_load_with_solver(self, solver):
+        """Specify the solver from the .load method."""
+
+        filename = os.path.join(TEST_FOLDER, "models", "simple1.json")
+        model = Model.load(filename, solver=solver)
+
+        for solver_class in pywr.solvers.solver_registry:
+            if solver_class.name == solver:
+                expected_solver = solver_class
+                break
+        else:
+            raise RuntimeError(f"Could not find solver with name: {solver}")
+
+        assert isinstance(model.solver, expected_solver)
+
+    @pytest.mark.parametrize("solver", ["null", "glpk", "glpk-edge"])
+    def test_load_with_solver_in_json(self, solver):
+        """Specify the solver in the JSON file."""
+
+        filename = os.path.join(TEST_FOLDER, "models", "simple1.json")
+        with open(filename) as fh:
+            data = json.load(fh)
+
+        data["solver"] = {
+            "name": solver,
+        }
+
+        model = Model.load(data)
+
+        for solver_class in pywr.solvers.solver_registry:
+            if solver_class.name == solver:
+                expected_solver = solver_class
+                break
+        else:
+            raise RuntimeError(f"Could not find solver with name: {solver}")
+
+        assert isinstance(model.solver, expected_solver)
+
+    @pytest.mark.parametrize("solver", ["null", "glpk", "glpk-edge"])
+    def test_load_override_solver_in_json(self, solver):
+        """Override the JSON definition with a keyword argument."""
+
+        filename = os.path.join(TEST_FOLDER, "models", "simple1.json")
+        with open(filename) as fh:
+            data = json.load(fh)
+
+        data["solver"] = {
+            "name": "glpk",
+        }
+
+        model = Model.load(data, solver=solver)
+
+        for solver_class in pywr.solvers.solver_registry:
+            if solver_class.name == solver:
+                expected_solver = solver_class
+                break
+        else:
+            raise RuntimeError(f"Could not find solver with name: {solver}")
+
+        assert isinstance(model.solver, expected_solver)
+
+    @pytest.mark.parametrize("solver", ["null", "glpk", "glpk-edge"])
+    def test_load_solver_in_env(self, solver):
+        """Specify the solver in an environment variable."""
+        filename = os.path.join(TEST_FOLDER, "models", "simple1.json")
+        with open(filename) as fh:
+            data = json.load(fh)
+
+        os.environ["PYWR_SOLVER"] = solver
+
+        model = Model.load(data)
+
+        for solver_class in pywr.solvers.solver_registry:
+            if solver_class.name == solver:
+                expected_solver = solver_class
+                break
+        else:
+            raise RuntimeError(f"Could not find solver with name: {solver}")
+
+        assert isinstance(model.solver, expected_solver)
+
+    @pytest.mark.parametrize("solver", ["null", "glpk", "glpk-edge"])
+    def test_load_override_solver_in_env(self, solver):
+        """Override the environment variables with a keyword argument."""
+        filename = os.path.join(TEST_FOLDER, "models", "simple1.json")
+        with open(filename) as fh:
+            data = json.load(fh)
+
+        os.environ["PYWR_SOLVER"] = "glpk"
+
+        model = Model.load(data, solver=solver)
+
+        for solver_class in pywr.solvers.solver_registry:
+            if solver_class.name == solver:
+                expected_solver = solver_class
+                break
+        else:
+            raise RuntimeError(f"Could not find solver with name: {solver}")
+
+        assert isinstance(model.solver, expected_solver)
