@@ -42,6 +42,7 @@ from pywr.parameters import (
     load_parameter,
     InterpolatedFlowParameter,
     ScenarioDailyProfileParameter,
+    RollingMeanFlowNodeParameter,
 )
 from pywr.recorders import AssertionRecorder, assert_rec
 from pywr.model import OrphanedParameterWarning
@@ -468,7 +469,6 @@ def test_daily_profile_leap_day(model):
 
 class TestScenarioDailyProfileParameter:
     def test_scenario_daily_profile(self, simple_linear_model):
-
         model = simple_linear_model
         scenario = Scenario(model, "A", 2)
         values = np.array(
@@ -509,7 +509,6 @@ class TestScenarioDailyProfileParameter:
 
 
 def test_scenario_weekly_profile(simple_linear_model):
-
     model = simple_linear_model
     scenario = Scenario(model, "A", 2)
 
@@ -547,7 +546,7 @@ def test_weekly_profile(simple_linear_model):
     @assert_rec(model, p)
     def expected_func(timestep, scenario_index):
         week = int(min((timestep.dayofyear - 1) // 7, 51))
-        value = week ** 2 + 27.5
+        value = week**2 + 27.5
         return value
 
     model.run()
@@ -557,7 +556,6 @@ class TestAnnualHarmonicSeriesParameter:
     """Tests for `AnnualHarmonicSeriesParameter`"""
 
     def test_single_harmonic(self, model):
-
         p1 = AnnualHarmonicSeriesParameter(model, 0.5, [0.25], [np.pi / 4])
         si = ScenarioIndex(0, np.array([0], dtype=np.int32))
 
@@ -583,7 +581,6 @@ class TestAnnualHarmonicSeriesParameter:
             np.testing.assert_allclose(p1.value(ts, si), expected)
 
     def test_load(self, model):
-
         data = {
             "type": "annualharmonicseries",
             "mean": 0.5,
@@ -629,7 +626,7 @@ class TestAnnualHarmonicSeriesParameter:
 
 
 def custom_test_func(array, axis=None):
-    return np.sum(array ** 2, axis=axis)
+    return np.sum(array**2, axis=axis)
 
 
 class TestAggregatedParameter:
@@ -787,7 +784,6 @@ class TestAggregatedIndexParameter:
 
 
 def test_parameter_child_variables(model):
-
     p1 = Parameter(model)
     # Default parameter
     assert len(p1.parents) == 0
@@ -953,7 +949,6 @@ def test_parameter_df_subsample():
 
 
 def test_parameter_df_json_load(model, tmpdir):
-
     # Daily time-step
     index = pd.date_range("2015-01-01", periods=365, freq="D", name="date")
     df = pd.DataFrame(np.random.rand(365), index=index, columns=["data"])
@@ -972,7 +967,6 @@ def test_parameter_df_json_load(model, tmpdir):
 
 
 def test_parameter_df_embed_load(model):
-
     # Daily time-step
     index = pd.date_range("2015-01-01", periods=365, freq="D", name="date")
     df = pd.DataFrame(np.random.rand(365), index=index, columns=["data"])
@@ -993,6 +987,32 @@ def test_parameter_df_embed_load(model):
     p.setup()
 
 
+@pytest.mark.parametrize("timestep_offset", [0, 1, -1, 2])
+def test_df_timestep_offset(timestep_offset):
+    """Check `timestep_offset` works as expected for `DataFrameParameter`"""
+    model = load_model("timeseries1.json")
+
+    # check first day initalised
+    assert model.timestepper.start == datetime.datetime(2015, 1, 1)
+
+    # check results
+    demand1 = model.nodes["demand1"]
+    catchment1 = model.nodes["catchment1"]
+    # Modify the timestep offset
+    catchment1.max_flow.timestep_offset = timestep_offset
+
+    expected_array = [23.92, 22.14, 22.57, 24.97, 27.59]
+    if timestep_offset > 0:
+        expected_array = expected_array[timestep_offset:]
+    elif timestep_offset < 0:
+        expected_array = [23.92] * abs(timestep_offset) + expected_array
+
+    for expected in expected_array:
+        result = model.step()
+        assert_allclose(catchment1.flow, expected, atol=1e-7)
+        assert_allclose(demand1.flow, min(expected, 23.0), atol=1e-7)
+
+
 def test_simple_json_parameter_reference():
     # note that parameters in the "parameters" section cannot be literals
     model = load_model("parameter_reference.json")
@@ -1195,7 +1215,7 @@ class Test1DPolynomialParameter:
 
         @assert_rec(model, p1)
         def expected_func(timestep, scenario_index):
-            return 0.5 + np.pi * x + 3.0 * x ** 2
+            return 0.5 + np.pi * x + 3.0 * x**2
 
         model.run()
 
@@ -1217,7 +1237,6 @@ class Test1DPolynomialParameter:
         # Test with proportional storage
         @assert_rec(model, p2, name="proportionalassertion")
         def expected_func(timestep, scenario_index):
-
             return 0.5 + np.pi * x / stg.max_volume
 
         model.setup()
@@ -1329,7 +1348,7 @@ class TestInterpolatedQuadratureParameter:
             if i < 0:
                 value = 0
             elif i < 6:
-                value = 2 * i ** 2 / 2
+                value = 2 * i**2 / 2
             elif i < 11:
                 value = 25 + 4 * (i - 5) ** 2 / 2 + (i - 5) * 10
             else:
@@ -1368,7 +1387,7 @@ class TestInterpolatedQuadratureParameter:
         def expected_func(timestep, scenario_index):
             i = timestep.index
             if i < 6:
-                value = 2 * i ** 2 / 2
+                value = 2 * i**2 / 2
             elif i < 11:
                 value = 25 + 4 * (i - 5) ** 2 / 2 + (i - 5) * 10
             else:
@@ -1632,7 +1651,7 @@ def test_ocptt(simple_linear_model):
     s2 = Scenario(model, "scenario 1", size=2)
     x = np.arange(len(model.timestepper)).reshape([len(model.timestepper), 1]) + 5
     y = np.arange(s1.size).reshape([1, s1.size])
-    z = x * y ** 2
+    z = x * y**2
     p = ArrayIndexedScenarioParameter(model, s1, z)
     inpt.max_flow = p
     model.setup()
@@ -1716,7 +1735,6 @@ class TestThresholdParameters:
         model.run()
 
     def test_multiple_threshold_index_parameter(self, simple_linear_model):
-
         model = simple_linear_model
         model.nodes["Input"].max_flow = ArrayIndexedParameter(model, np.arange(0, 20))
         model.nodes["Output"].cost = -10.0
@@ -1746,7 +1764,6 @@ class TestThresholdParameters:
         model.run()
 
     def test_multiple_threshold_parameter_index_parameter(self, simple_linear_model):
-
         model = simple_linear_model
         model.nodes["Input"].max_flow = ArrayIndexedParameter(
             model, np.arange(0, 20), name="max_flow"
@@ -1945,6 +1962,33 @@ def test_flow_parameter():
     assert_allclose(expected_yesterday, actual_yesterday[:, 0])
 
 
+@pytest.mark.parametrize("use_proportional_volume", [None, True, False])
+def test_storage_parameter(use_proportional_volume: bool):
+    """test StorageParameter"""
+    model = load_model("storage_parameter.json")
+
+    if use_proportional_volume is not None:
+        model.parameters["storage"].use_proportional_volume = use_proportional_volume
+
+    model.run()
+
+    if model.parameters["storage"].use_proportional_volume:
+        expected_volume = np.array(
+            [34.900000, 34.800286, 34.700856, 34.601711, 34.502849, 34.404269]
+        )
+        expected_demand = np.array(
+            [0.100000, 0.099714, 0.099429, 0.099145, 0.098862, 0.098580]
+        )
+    else:
+        expected_volume = np.array([31.5, 28.35, 25.515, 22.9635, 20.66715, 18.600435])
+        expected_demand = np.array([3.5, 3.15, 2.835, 2.5515, 2.29635, 2.066715])
+
+    actual_demand = model.recorders["flow_recorder"].data
+    actual_volume = model.recorders["storage_recorder"].data
+    assert_allclose(expected_demand, actual_demand[:, 0], rtol=1e-4)
+    assert_allclose(expected_volume, actual_volume[:, 0], rtol=1e-4)
+
+
 class TestHydroPowerTargets:
     def test_target_json(self):
         """Test loading a HydropowerTargetParameter from JSON."""
@@ -2215,3 +2259,73 @@ class TestDiscountFactorParameter:
             return 1 / pow(1.035, year - 2015)
 
         model.run()
+
+
+class TestRollingMeanFlowNodeParameter:
+    @pytest.mark.parametrize(
+        "initial_flow",
+        [0.0, 2.0],
+    )
+    def test_mean_flow_recorder(self, model, initial_flow):
+        model.timestepper.start = pd.to_datetime("2016-01-01")
+        model.timestepper.end = pd.to_datetime("2016-01-04")
+
+        inpt = Input(model, "input")
+        otpt = Output(model, "output")
+        inpt.connect(otpt)
+
+        p_mean = RollingMeanFlowNodeParameter(
+            model, node=inpt, timesteps=3, initial_flow=initial_flow
+        )
+
+        _scenario = Scenario(model, "dummy", size=2)
+
+        inpt.max_flow = inpt.min_flow = FunctionParameter(
+            model, inpt, lambda model, t, si: 2 + t.index
+        )
+
+        expected = [
+            initial_flow,
+            2.0,
+            (2.0 + 3.0) / 2,
+            (2.0 + 3.0 + 4.0) / 3,
+            (3.0 + 4.0 + 5.0) / 3,  # zeroth day forgotten
+        ]
+
+        @assert_rec(model, p_mean)
+        def expected_func(timestep, scenario_index):
+            return expected[timestep.index]
+
+        model.run()
+
+    def test_mean_flow_recorder_days(self, model):
+        model.timestepper.delta = 7
+
+        inpt = Input(model, "input")
+        otpt = Output(model, "output")
+        inpt.connect(otpt)
+
+        p_mean = RollingMeanFlowNodeParameter(model, node=inpt, days=31)
+
+        model.run()
+        assert p_mean.timesteps == 4
+
+    def test_mean_flow_recorder_json(self, model):
+        model = load_model("mean_flow_parameter.json")
+
+        supply1 = model.nodes["supply1"]
+        supply1.max_flow = supply1.min_flow = FunctionParameter(
+            model, supply1, lambda model, t, si: 2 + t.index
+        )
+
+        assert len(model.recorders) == 3
+
+        rec_flow = model.recorders["Supply"]
+        rec_mean = model.recorders["Mean Flow"]
+        rec_check = model.recorders["Supply 2"]
+
+        model.run()
+
+        assert_allclose(rec_flow.data[:, 0], [2.0, 3.0, 4.0, 5.0])
+        assert_allclose(rec_mean.data[:, 0], [0.0, 2.0, 2.5, 3.0])
+        assert_allclose(rec_check.data[:, 0], [50.0, 50.0, 60.0, 60.0])
