@@ -685,7 +685,6 @@ def test_numpy_array_level_recorder(simple_storage_model):
 
 
 def test_numpy_array_area_recorder(simple_storage_model):
-
     model = simple_storage_model
 
     storage = model.nodes["Storage"]
@@ -903,7 +902,6 @@ class TestTotalParameterRecorder:
 
     @pytest.mark.parametrize("integrate", [None, True, False])
     def test_from_json(self, daily_profile_model, integrate):
-
         model = daily_profile_model
 
         data = {
@@ -938,7 +936,6 @@ class TestMeanParameterRecorder:
         assert_allclose(rec.values(), expected)
 
     def test_from_json(self, daily_profile_model):
-
         model = daily_profile_model
 
         data = {
@@ -1232,6 +1229,43 @@ class TestTablesRecorder:
                 assert row["A"] == comb[0]
                 assert row["B"] == comb[1]
 
+    def test_scenario_slices(self, simple_linear_model, tmpdir):
+        """
+        Test the TablesRecorder with user defined scenario subset
+
+        """
+        from pywr.parameters import ConstantScenarioParameter
+
+        model = simple_linear_model
+        scA = Scenario(model, name="A", size=4, slice=slice(1, 4))
+        scB = Scenario(model, name="B", size=2)
+
+        otpt = model.nodes["Output"]
+        inpt = model.nodes["Input"]
+
+        inpt.max_flow = ConstantScenarioParameter(model, scA, [10, 20, 30, 40])
+        otpt.max_flow = ConstantScenarioParameter(model, scB, [20, 40])
+        otpt.cost = -2.0
+
+        h5file = tmpdir.join("output.h5")
+        import tables
+
+        with tables.open_file(str(h5file), "w") as h5f:
+            rec = TablesRecorder(model, h5f)
+
+            model.run()
+
+            for node_name in model.nodes.keys():
+                ca = h5f.get_node("/", node_name)
+                assert ca.shape == (365, 3, 2)
+
+            # check slices table exists
+            slices = h5f.get_node("/scenario_slices")
+            assert slices[0]["name"] == b"A"
+            assert slices[0]["start"] == 1
+            assert slices[0]["stop"] == 4
+            assert slices[0]["step"] == 1
+
     def test_parameters(self, simple_linear_model, tmpdir):
         """
         Test the TablesRecorder
@@ -1337,7 +1371,6 @@ class TestTablesRecorder:
         import tables
 
         with tables.open_file(str(h5file), "w") as h5f:
-
             nodes = [
                 ("/outputs/demand", "Demand"),
                 ("/storage/reservoir", "Reservoir"),
@@ -1352,8 +1385,8 @@ class TestTablesRecorder:
             model.run()
 
             max_volume = model.nodes["Reservoir"].max_volume
-            rec_demand = h5f.get_node("/outputs/demand", "Demand").read()
-            rec_storage = h5f.get_node("/storage/reservoir", "Reservoir").read()
+            rec_demand = h5f.get_node("/outputs/demand").read()
+            rec_storage = h5f.get_node("/storage/reservoir").read()
 
             # model starts with no demand saving
             demand_baseline = 50.0
@@ -1377,7 +1410,7 @@ class TestTablesRecorder:
                 rec_demand[12, 0], demand_baseline * demand_factor * demand_saving
             )
 
-    def test_demand_saving_with_indexed_array(self, tmpdir):
+    def test_demand_saving_with_indexed_array_from_json(self, tmpdir):
         """Test recording various items from demand saving example.
 
         This time the TablesRecorder is defined in JSON.
@@ -1783,15 +1816,16 @@ def test_timestep_count_index_parameter_recorder(simple_storage_model):
 
 
 @pytest.mark.parametrize(
-    ("params", "exclude_months"),
+    ("params", "exclude_months", "include_range"),
     [
-        [1, None],
-        [2, None],
-        [1, [1, 2, 12]],
+        [1, None, [None, None, None, None]],
+        [2, None, [None, None, None, None]],
+        [1, [1, 2, 12], [None, None, None, None]],
+        [1, None, [4, 1, 8, 15]],
     ],
 )
 def test_annual_count_index_threshold_recorder(
-    simple_storage_model, params, exclude_months
+    simple_storage_model, params, exclude_months, include_range
 ):
     """
     The test sets uses a simple reservoir model with different inputs that
@@ -1819,18 +1853,43 @@ def test_annual_count_index_threshold_recorder(
         model, scenario, [demand, 0]
     )
     model.nodes["Output"].max_flow = demand
+    (
+        include_from_month,
+        include_from_day,
+        include_to_month,
+        include_to_day,
+    ) = include_range
 
+    # import pdb; pdb.set_trace()
     # Create the recorder with a threshold of 1
     rec = AnnualCountIndexThresholdRecorder(
-        model, [param] * params, "TestRec", 1, exclude_months=exclude_months
+        model,
+        [param] * params,
+        "TestRec",
+        1,
+        exclude_months=exclude_months,
+        include_from_month=include_from_month,
+        include_from_day=include_from_day,
+        include_to_day=include_to_day,
+        include_to_month=include_to_month,
     )
 
     model.run()
 
     # We expect no failures in the first ensemble, the reservoir starts failing halfway through
     # the 3rd year
-    if exclude_months is None:
+    if exclude_months is None and None in include_range:
         expected_data = [[0, 0], [0, 0], [0, 183], [0, 365], [0, 365]]
+    elif None not in include_range:
+        # Ignore counts between the 1st April and 15th Aug inclusive
+        assert include_range == [4, 1, 8, 15]
+        expected_data = [
+            [0, 0],
+            [0, 0],
+            [0, 30 + 15],
+            [0, 30 + 31 + 30 + 31 + 15],
+            [0, 30 + 31 + 30 + 31 + 15],
+        ]
     else:
         # Ignore counts for Jan, Feb and Dec
         assert exclude_months == [
@@ -1845,7 +1904,6 @@ def test_annual_count_index_threshold_recorder(
             [0, 365 - 31 - 28 - 31],
             [0, 365 - 31 - 28 - 31],
         ]
-
     assert_allclose(expected_data, rec.data, atol=1e-7)
     df = rec.to_dataframe()
     assert_allclose(expected_data, df.values, atol=1e-7)
@@ -1964,7 +2022,7 @@ def test_mean_flow_node_recorder(simple_linear_model):
 
 
 def custom_test_func(array, axis=None):
-    return np.sum(array ** 2, axis=axis)
+    return np.sum(array**2, axis=axis)
 
 
 class TestAggregatedRecorder:
