@@ -3,6 +3,8 @@
 
 import os
 import datetime
+
+import numpy as np
 import pytest
 import pandas
 from numpy.testing import assert_allclose
@@ -14,6 +16,7 @@ import pywr.solvers
 import pywr.parameters.licenses
 import pywr.domains.river
 from pywr.recorders import assert_rec
+from pywr.parameters import Parameter, ConstantParameter
 from helpers import load_model
 
 import pywr.parameters
@@ -101,7 +104,6 @@ def test_run_reservoir2():
 
 
 def test_empty_storage_min_flow():
-
     model = Model()
     storage = Storage(
         model, "storage", initial_volume=100, max_volume=100, inputs=1, outputs=0
@@ -438,6 +440,145 @@ class TestRollingVirtualStorage:
         assert_allclose(otpt.flow, [10.0], atol=1e-7)
         assert_allclose(vs.volume, [30.0], atol=1e-7)
 
+    def test_initial_volume(self):
+        """Test RollingVirtualStorage node behaviour when initial volume is less than max volume"""
+
+        model = pywr.core.Model(timestep="1D")
+
+        inpt = Input(model, "Input", max_flow=0.0)
+        lnk = Link(model, "Link")
+        inpt.connect(lnk)
+        otpt = Output(model, "Output", max_flow=10, cost=-10.0)
+        lnk.connect(otpt)
+
+        vs = pywr.core.RollingVirtualStorage(
+            model, "Licence", [lnk], timesteps=3, initial_volume=70.0, max_volume=100.0
+        )
+
+        model.setup()
+
+        assert_allclose(vs.volume, [70.0], atol=1e-7)
+
+        model.step()
+
+        assert_allclose(otpt.flow, [0.0], atol=1e-7)
+        assert_allclose(vs.volume, [85.0], atol=1e-7)
+
+        model.step()
+
+        assert_allclose(otpt.flow, [0.0], atol=1e-7)
+        assert_allclose(vs.volume, [100.0], atol=1e-7)
+
+        model.step()
+
+        assert_allclose(otpt.flow, [0.0], atol=1e-7)
+        assert_allclose(vs.volume, [100.0], atol=1e-7)
+
+    def test_initial_pc(self):
+        """Test RollingVirtualStorage node behaviour when initial volume percentage is less than 1.0"""
+
+        model = pywr.core.Model(timestep="1D")
+
+        inpt = Input(model, "Input", max_flow=0.0)
+        lnk = Link(model, "Link")
+        inpt.connect(lnk)
+        otpt = Output(model, "Output", max_flow=10, cost=-10.0)
+        lnk.connect(otpt)
+
+        mx_vol = ConstantParameter(model, 100)
+        vs = pywr.core.RollingVirtualStorage(
+            model,
+            "Licence",
+            [lnk],
+            timesteps=3,
+            initial_volume=70.0,
+            initial_volume_pc=0.7,
+            max_volume=mx_vol,
+        )
+
+        model.setup()
+
+        assert_allclose(vs.volume, [70.0], atol=1e-7)
+
+        model.step()
+
+        assert_allclose(otpt.flow, [0.0], atol=1e-7)
+        assert_allclose(vs.volume, [85.0], atol=1e-7)
+
+        model.step()
+
+        assert_allclose(otpt.flow, [0.0], atol=1e-7)
+        assert_allclose(vs.volume, [100.0], atol=1e-7)
+
+        model.step()
+
+        assert_allclose(otpt.flow, [0.0], atol=1e-7)
+        assert_allclose(vs.volume, [100.0], atol=1e-7)
+
+    def test_initial_volume_zero(self):
+        """Test RollingVirtualStorage node behaviour when initial volume is zero"""
+
+        model = pywr.core.Model(timestep="1D")
+
+        inpt = Input(model, "Input", max_flow=0.0)
+        lnk = Link(model, "Link")
+        inpt.connect(lnk)
+        otpt = Output(model, "Output", max_flow=10, cost=-10.0)
+        lnk.connect(otpt)
+
+        vs = pywr.core.RollingVirtualStorage(
+            model,
+            "Licence",
+            [lnk],
+            timesteps=3,
+            initial_volume=0.0,
+            max_volume=100.0,
+        )
+
+        model.setup()
+
+        assert_allclose(vs.volume, [0.0], atol=1e-7)
+
+        model.step()
+
+        assert_allclose(otpt.flow, [0.0], atol=1e-7)
+        assert_allclose(vs.volume, [50.0], atol=1e-7)
+
+        model.step()
+
+        assert_allclose(otpt.flow, [0.0], atol=1e-7)
+        assert_allclose(vs.volume, [100.0], atol=1e-7)
+
+        model.step()
+
+        assert_allclose(otpt.flow, [0.0], atol=1e-7)
+        assert_allclose(vs.volume, [100.0], atol=1e-7)
+
+    def test_max_vol_param_zero_initial_vol(self):
+        """Test RollingVirtualStorage node behaviour when initial volume is zero"""
+
+        model = pywr.core.Model(timestep="1D")
+
+        inpt = Input(model, "Input", max_flow=0.0)
+        lnk = Link(model, "Link")
+        inpt.connect(lnk)
+        otpt = Output(model, "Output", max_flow=10, cost=-10.0)
+        lnk.connect(otpt)
+
+        mx_vol = ConstantParameter(model, 100)
+        pywr.core.RollingVirtualStorage(
+            model,
+            "Licence",
+            [lnk],
+            timesteps=3,
+            initial_volume=0.0,
+            initial_volume_pc=0.0,
+            max_volume=mx_vol,
+        )
+
+        with pytest.raises(ValueError):
+            model.setup()
+
 
 @pytest.mark.parametrize(
     "test_model", ["virtual_storage1", "virtual_storage6", "virtual_storage7"]
@@ -495,7 +636,7 @@ def test_annual_virtual_storage_with_dynamic_cost():
 
 
 @pytest.mark.xfail(
-    raises=AssertionError,
+    raises=pywr.solvers.GLPKError,
     reason="See issue #1001: https://github.com/pywr/pywr/issues/1001",
 )
 def test_annual_virtual_storage_with_piecewise_link():
@@ -515,7 +656,7 @@ def test_annual_virtual_storage_with_piecewise_link():
 
 
 @pytest.mark.xfail(
-    raises=AssertionError,
+    raises=pywr.solvers.GLPKError,
     reason="See issue #1001: https://github.com/pywr/pywr/issues/1001",
 )
 def test_annual_virtual_storage_with_aggregated_node():
@@ -536,7 +677,6 @@ def test_annual_virtual_storage_with_aggregated_node():
 
 class TestSeasonalVirtualStorage:
     def test_run(self):
-
         model = load_model("seasonal_virtual_storage.json")
         model.run()
         supply_df = model.recorders["supply1"].to_dataframe()
@@ -623,6 +763,60 @@ class TestSeasonalVirtualStorage:
         assert_allclose(supply_df.loc["2015-03-01", :], 10)
 
 
+class TestMonthlyVirtualStorage:
+    def test_simple(self):
+        """Test MonthlyVirtualStorage with `Months` set to 3"""
+        model = load_model("virtual_storage8.json")
+        model.run()
+        df = model.recorders["supply1"].to_dataframe()
+
+        # flow on first day should be 10.0
+        assert df.loc["2015-01-01", 0].squeeze() == 10.0
+        # Licence should be exhausted after 30 days
+        assert df.loc["2015-01-31", 0].squeeze() == 0.0
+        # Licence should  still be exhausted at end of march
+        assert df.loc["2015-03-31", 0].squeeze() == 0.0
+        # Licence should be reset at start of April
+        assert df.loc["2015-04-01", 0].squeeze() == 10.0
+        # Licence should  be exhausted at end of June
+        assert df.loc["2015-06-30", 0].squeeze() == 0.0
+        # Licence should be reset at start of July
+        assert df.loc["2015-07-01", 0].squeeze() == 10.0
+
+    def test_initial_months(self):
+        """Test MonthlyVirtualStorage with `months` set to 3 and `initial_months` set to 1"""
+        model = load_model("virtual_storage8.json")
+        model.nodes["licence1"].initial_months = 1
+        model.run()
+        df = model.recorders["supply1"].to_dataframe()
+
+        # flow on first day should be 10.0
+        assert df.loc["2015-01-01", 0].squeeze() == 10.0
+        # Licence should be exhausted after 30 days
+        assert df.loc["2015-01-31", 0].squeeze() == 0.0
+        # Licence should  still be exhausted at end of feb
+        assert df.loc["2015-02-28", 0].squeeze() == 0.0
+        # Licence should be reset at start of March
+        assert df.loc["2015-03-01", 0].squeeze() == 10.0
+
+    def test_longer_licences(self):
+        """Test MonthlyVirtualStorage with `months` set to 13"""
+        model = load_model("virtual_storage8.json")
+        model.timestepper.end = "2017-03-02"
+        model.nodes["licence1"].months = 13
+        model.run()
+        df = model.recorders["supply1"].to_dataframe()
+
+        # flow on first day should be 10.0
+        assert df.loc["2015-01-01", 0].squeeze() == 10.0
+        # Once licence is exhausted it should not be reset until Feb the following year
+        assert not np.any(df.loc["2015-01-31":"2016-01-31", 0].values)
+        assert df.loc["2016-02-01", 0].squeeze() == 10.0
+        # Once licence is exhausted it should not be reset until March the following year
+        assert not np.any(df.loc["2016-03-02":"2017-02-28", 0].values)
+        assert df.loc["2017-03-01", 0].squeeze() == 10.0
+
+
 def test_storage_spill_compensation():
     """Test storage spill and compensation flows
 
@@ -836,6 +1030,17 @@ def test_run_empty():
         model.run()
 
 
+@pytest.mark.parametrize(
+    "filename", ["simple1_broken.json", "simple1_semi_broken.json"]
+)
+def test_broken_model(filename):
+    """Model with no valid routes should not work."""
+    # Load, but don't trigger a `Model.check()` to make sure that the solver gives a useful error message
+    model = load_model(filename, check=False)
+    with pytest.raises(ModelStructureError):
+        model.run()
+
+
 def test_run():
     model = load_model("simple1.json")
 
@@ -918,3 +1123,75 @@ def test_select_glpk_presolve(use_presolve):
         model = load_model(data=data)
         assert model.solver.name.lower() == solver_name
         assert model.solver._cy_solver.use_presolve == (use_presolve == "true")
+
+
+class NanParameter(Parameter):
+    """A parameter that returns a NaN for testing error handling"""
+
+    def value(self, ts, si):
+        return np.NAN
+
+
+class TestGlpkErrorHandling:
+    @pytest.mark.skipif(
+        Model().solver.name == "lpsolve" or Model().solver.use_unsafe_api,
+        reason="NaN not checked for lpsolve or unsafe GLPK API.",
+    )
+    def test_nan_constraint_error(self):
+        """Test a NaN in a row constraint causes an error"""
+        # parse the JSON into a model
+        model = load_model("simple1.json")
+
+        nan_param = NanParameter(model)
+        inpt = model.nodes["supply1"]
+        inpt.max_flow = nan_param
+
+        model.setup()
+
+        with pytest.raises(pywr.solvers.GLPKError):
+            model.run()
+
+    @pytest.mark.skipif(
+        Model().solver.name == "lpsolve" or Model().solver.use_unsafe_api,
+        reason="NaN not checked for lpsolve or unsafe GLPK API.",
+    )
+    def test_nan_cost_error(self):
+        """Test a NaN in a node cost causes an error"""
+        # parse the JSON into a model
+        model = load_model("simple1.json")
+
+        nan_param = NanParameter(model)
+        inpt = model.nodes["supply1"]
+        inpt.cost = nan_param
+
+        model.setup()
+
+        with pytest.raises(pywr.solvers.GLPKError):
+            model.run()
+
+
+def test_setup_profiler(tmp_path):
+    """Test the setup resource profiler."""
+    model = load_model("demand_saving2.json")
+
+    profile_out = tmp_path / "stats.csv"
+    model.setup(profile=True, profile_dump_filename=profile_out)
+
+    assert profile_out.exists()
+    df = pandas.read_csv(profile_out)
+    # 4 nodes & 10 parameters for setup & reset
+    # Plus reservoir has 2 internal nodes that are setup
+    assert len(df) == 14 * 2 + 2
+
+
+def test_reset_profiler(tmp_path):
+    """Test the reset resource profiler."""
+    model = load_model("demand_saving2.json")
+
+    profile_out = tmp_path / "stats.csv"
+    model.setup()
+    model.reset(profile=True, profile_dump_filename=profile_out)
+
+    assert profile_out.exists()
+    df = pandas.read_csv(profile_out)
+    assert len(df) == 14  # 4 nodes & 10 parameters for reset
